#include "LPC18xx.h"            
#include "config.h"
#include "app_setup.h"
#include "reloc_irq_table.h"

static void vIOInit(void);

uint32_t t;
#define DelayNOPS(x)  for(t=0;t<x;t++)__NOP();
#ifdef EXT_FLASH
static void vFlashConfig(void);
#endif

void vAppInit(void)
{
	F6_10 f6_10tmp;
	UART_Init_Type UART0, UART1;
	
	/* Init system */
	SystemInit();
		
#ifdef EXT_FLASH
	/* relocate vector table to internal ram */
	relocIrqTable(); 
	/* Configure the flash for full 16-bit adressing */
	vFlashConfig();
#endif

	/* Setup clocking */ 
    SetPL160M(SRC_XTAL, 10);						// 12*10=120 MHz		
	SetClock(BASE_M3_CLK, SRC_PL160M_0, DIV1);		// Run base M3 clock from PL160M, no division	
 	SetClock(BASE_UART0_CLK, SRC_PL160M_0, DIV1);	// Run UART0 clk from PL160M, no division 
	SetClock(BASE_UART1_CLK, SRC_PL160M_0, DIV1);	// Run UART1 clk from PL160M, no division 
	SetClock(BASE_VPB3_CLK, SRC_PL160M_0, DIV1);	// base_vpb3_clk ; vpb1_adcx_clk 
	SetClock(BASE_M0_CLK, SRC_PL160M_0, DIV1);		// Run base M0 clock from XTAL, div by 1 = no division
	SysTick_Config(M3Frequency/100);  				// Generate interrupt @ 100 Hz    		

	/* Configure the IO's */
	vIOInit();			
 
	/* Initialize peripherals */
    UART0.portNum = 0;			   					// RS232: used for MC comms 
	UART0.baudrate = 115200;
	UART0.rx_callback = vMC_COMMS_ProcessChar;
    u32UARTInit(&UART0);							

	UART1.portNum = 1;					   			// USB VCP: used for debugging with printf 
	UART1.baudrate = 115200;
	UART1.rx_callback = NULL;
	u32UARTInit(&UART1);							

    vQEI_Init();
	f6_10tmp.full=QEI_CALIB_DUTY;
	vCALIB_InitQEICalib(f6_10tmp, QEI_CALIB_DELAY);

	// Phase channels defined in app_setup.h
	// Phase A = ADC0, CH3 
	// Phase B = ADC0, CH4
	// Phase C = ADC1, CH5
	// BUS currrent = ADC1, CH6
	vADC0Init((1<<ADC_PHA_CH) | (1<<ADC_PHB_CH), 4444444,  ADC_START_CTOUT_8, ADC_EDGE_RISING);	 // 4.44MHz ADC 
	vADC1Init((1<<ADC_PHC_CH) | (1<<ADC_IBUS_CH), 4444444,  ADC_START_CTOUT_8, ADC_EDGE_RISING); // 4.44MHz ADC 
	// ADC0 is triggered at same time as ADC1. Therefore ADC0 CH4 will 
	// last finish conversion. When the callback is executed all the 
	// ADC DR's will contain all the adc sample value's.
	vADC_SetChCallback(0, 4, (ADC_CH_CALLBACK)vMC_FOC_Loop);

	vSCT_Init();   
	vDMA_Init();
	vDMA_SetChCallback(0, (DMA_CH_CALLBACK)vMC_COMMS_SetTransmissionComplete);							
}

/*****************************************************************************
** Function name:		Application specific 
**
** Description:			100 ms Call function
**
** Parameters:			None
** Returned value:		None
**
*****************************************************************************/
void vAppSetDefault(MC_STORAGE_Type *stored)
{
	uint8_t i;
	
	/* Set CTRL default values */
	stored->ctrl.type = vMC_LIB_GetMCType();
	stored->ctrl.sensor_mode = MODE_QEI;

	stored->ctrl.pidSpeed.sp.full = i32RPM_TO_F6_10PID(500);	// 500 RPM 
	stored->ctrl.pidSpeed.kp.full = F6_10_CONST(0.06);		
	f6_10MC_LIB_PIDSetKi(F6_10_CONST(0.003), &stored->ctrl.pidSpeed);
	stored->ctrl.pidSpeed.kd.full = F6_10_CONST(0.0);

	/* Set CALIB default values */
	stored->calib.QEIoffset = QEI_OFFSET;				
	stored->calib.QEIinvert = QEI_INVERT;
	stored->calib.QEIuseinx = QEI_USE_INDEX;
	
	for(i=0;i<3;i++)
	{
		stored->calib.IphADCpoint[i] = PWM_ADC_POINT;	
		stored->calib.IphADCgain[i].full = ADC_PHASE_GAIN;	
	}

	stored->calib.IphADCoffset[0] = ADC_PHA_OFFSET;	
	stored->calib.IphADCoffset[1] = ADC_PHB_OFFSET;	
	stored->calib.IphADCoffset[2] = ADC_PHC_OFFSET;	

	stored->calib.IbusADCoffset = I_BUS_OFFSET;
	stored->calib.IbusADCgain.full = I_BUS_GAIN;
	stored->calib.VbusADCoffset = V_BUS_OFFSET;
	stored->calib.VbusADCgain.full = V_BUS_GAIN;

	/* Set FOC specific default values */
	stored->var.foc.pidQ.sp.full = F6_10_CONST(0.1);		
	stored->var.foc.pidQ.kp.full = F6_10_CONST(0.3);	
	f6_10MC_LIB_PIDSetKi(F6_10_CONST(0.1), &stored->var.foc.pidQ);
	stored->var.foc.pidQ.kd.full = F6_10_CONST(0.0);		

	stored->var.foc.pidD.sp.full = F6_10_CONST(0.0); 		
	stored->var.foc.pidD.kp.full = F6_10_CONST(0.2);	
	f6_10MC_LIB_PIDSetKi(F6_10_CONST(0.06), &stored->var.foc.pidD);
	stored->var.foc.pidD.kd.full = F6_10_CONST(0.0);			 	
}

static void vIOInit(void)
{
    /* Set LEDs pinmux */
	scu_pinmux(0x8 ,7 , MD_PDN, FUNC0); 	// P8.7 : GPIO4_7: LED1	
	scu_pinmux(0x8 ,6 , MD_PDN, FUNC0); 	// P8.6 : GPIO4_6: LED2	
	scu_pinmux(0x8 ,5 , MD_PDN, FUNC0); 	// P8.5 : GPIO4_5: LED3	
	scu_pinmux(0x8 ,4 , MD_PDN, FUNC0); 	// P8.4 : GPIO4_4: LED4	
	scu_pinmux(0x8 ,3,  MD_PDN, FUNC0); 	// P8.3 : GPIO4_3: LED5	
	// Cannot use LED6-8; connected to HALL sensors
	// scu_pinmux(0x8 ,2 , MD_PDN, FUNC0); 	// P8.2 : GPIO4_2: LED6	
	// scu_pinmux(0x8 ,1 , MD_PDN, FUNC0); 	// P8.1 : GPIO4_1: LED7	
	// scu_pinmux(0x8 ,0 , MD_PDN, FUNC0); 	// P8.0 : GPIO4_0: LED8	
    LPC_GPIO4->DIR |= 0xFF; 				// LEDs 1 to 8 = output 

	/* Set buttons pinmux */
	scu_pinmux(0x6 ,10 , MD_PLN|MD_EZI, FUNC0);	// SW2 = p6.10 = GPIO3_6 FUNC0
	scu_pinmux(0x4 ,0  , MD_PLN|MD_EZI, FUNC0);	// SW3 = p4.0  = GPIO2_0 FUNC0
	scu_pinmux(0xE ,2  , MD_PLN|MD_EZI, FUNC4);	// SW4 = pE.2  = GPIO7_2 FUNC4
	scu_pinmux(0x2 ,5  , MD_PLN|MD_EZI, FUNC4);	// SW5 = P2.5  = GPIO5_5 FUNC4
	LPC_GPIO3->DIR &= ~(1<<6);					// SW2 = input
	LPC_GPIO2->DIR &= ~(1<<0);					// SW3 = input
	LPC_GPIO7->DIR &= ~(1<<2);					// SW4 = input
	LPC_GPIO5->DIR &= ~(1<<5);					// SW5 = input
	
	/* MC control lines */
	scu_pinmux(0x7 ,7 , MD_PDN, FUNC0); 		// P7.7: GPIO3_15: MC_SEL_H
	scu_pinmux(0x7 ,6 , MD_PDN, FUNC0); 		// P7.6: GPIO3_14: MC_SEL_L
	scu_pinmux(0x7 ,5 , MD_PDN, FUNC0); 		// P7.5: GPIO3_13: SCT_SEL_H
	scu_pinmux(0x7 ,4 , MD_PDN, FUNC0); 		// P7.4: GPIO3_12: SCT_SEL_L
	scu_pinmux(0x7 ,3 , MD_PDN|MD_EZI, FUNC0); 	// P7.3 : GPIO3_11: CURR_TRIP 
	scu_pinmux(0x7 ,0 , MD_PDN, FUNC0); 		// P7.0 : GPIO3_8: CURR_TRIP_RST
 	scu_pinmux(0xD ,10 , MD_PDN|MD_EZI, FUNC4); // PD.10 : GPIO6_24: E-BREAK-STS0
	scu_pinmux(0xD ,13 , MD_PDN|MD_EZI, FUNC4); // PD.13 : GPIO6_27: E-BREAK-STS1
	scu_pinmux(0xA ,4 , MD_PDN, FUNC1); 		// PA.4 : SCT MAT0.9: E-BREAK-CTRL 
 	LPC_GPIO3->DIR |= (1<<15)|(1<<14)|(1<<13)|(1<<12)|(1<<8);	// Output
	LPC_GPIO6->DIR &= ~((1<<24)|(1<<27));						// Input
	LPC_GPIO3->DIR &= ~(1<<11);									// Input

	/* Enable high level interrupt on CURR_TRIP */
	/* Select P7_3: GPIO3_11 for pin interrupt */
	/* Now handled in main.c */	
//	LPC_SCU->PINTSEL0 |= (11<<0) | (3<<5); 
//	LPC_GPIO_PIN_INT->ISEL |= (1<<0);				// Level sensitive on pin 0 interrupt
//	LPC_GPIO_PIN_INT->SIENR = (1<<0);				// Enable rising edge or level interrupt on pin 0 interrupt
//	LPC_GPIO_PIN_INT->SIENF = (1<<0);				// Select HIGH-active interrupt on pin 0 interrupt
//	NVIC_EnableIRQ(PIN_INT0_IRQn);	
//
//	/* Enable high level interrupt on E-BREAK-STS0: Braking threshold reached */
//	/* Select PD_10: GPIO6_24 for pin interrupt */	
//	LPC_SCU->PINTSEL0 |= (24<<13) | (6<<16); 
//	LPC_GPIO_PIN_INT->ISEL |= (1<<1);				// Level sensitive on pin 0 interrupt
//	LPC_GPIO_PIN_INT->SIENR = (1<<1);				// Enable rising edge or level interrupt on pin 0 interrupt
//	LPC_GPIO_PIN_INT->SIENF = (1<<1);				// Select HIGH-active interrupt on pin 0 interrupt
//	NVIC_EnableIRQ(PIN_INT1_IRQn);	
//
//	/* Enable high level interrupt on E-BREAK-STS1: Braking forced by hardware */
//	/* Select PD_13: GPIO6_27 for pin interrupt */	
//	LPC_SCU->PINTSEL0 |= (27<<21) | (6<<24); 
//	LPC_GPIO_PIN_INT->ISEL |= (1<<2);				// Level sensitive on pin 0 interrupt
//	LPC_GPIO_PIN_INT->SIENR = (1<<2);				// Enable rising edge or level interrupt on pin 0 interrupt
//	LPC_GPIO_PIN_INT->SIENF = (1<<2);				// Select HIGH-active interrupt on pin 0 interrupt
//	NVIC_EnableIRQ(PIN_INT2_IRQn);	

	vAppResetCurrTrip();

	/* Set MC_SEL high: disable MCPWM selection */
	LPC_GPIO3->SET = (1<<15)|(1<<14);
	/* Set SCT_SEL low: enable SCT selection */
	LPC_GPIO3->CLR = (1<<13)|(1<<12);	

   	/* Pin configuration for SCT Timer */
	scu_pinmux (4, 2, MD_PLN, FUNC1);   	// P4_2:  SCTOUT0_0
	scu_pinmux (4, 1, MD_PLN, FUNC1);   	// P4_1:  SCTOUT0_1
	scu_pinmux (4, 4, MD_PLN, FUNC1);   	// P4_4:  SCTOUT0_2
	scu_pinmux (4, 3, MD_PLN, FUNC1);   	// P4_3:  SCTOUT0_3
	scu_pinmux (4, 6, MD_PLN, FUNC1);   	// P4_6:  SCTOUT0_4
	scu_pinmux (4, 5, MD_PLN, FUNC1);   	// P4_5:  SCTOUT0_5
}

/**********************************************************************
 ** Function name:	vAppResetCurrTrip	
 **
 ** Description: Resets current trip 		
 ** Parameters: None
 ** Returned value:	None
 **********************************************************************/
void vAppResetCurrTrip(void)	
{
	LPC_GPIO3->SET = (1<<8);
	LPC_GPIO3->CLR = (1<<8);
}

/**********************************************************************
 ** Function name:	vFlashConfig	
 **
 ** Description: Configure EMC 16 bit		
 **		Setup the external memory controller for 16-bit flash on CS0 
 ** 	By default the bootloader only itinitalises the first 5 adress lines
 ** 	Make sure that all the address lines for 16-bit are enabled. 
 ** Parameters: None
 **
 ** Returned value: None		
 **********************************************************************/
#ifdef EXT_FLASH
static void vFlashConfig(void)
{
 	scu_pinmux(1,7, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,8, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,9, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,10, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,11, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,12, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,13, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,14, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(5,4, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(5,5, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(5,6, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(5,7, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(5,0, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(5,1, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(5,2, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(5,3, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,2, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,3, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,4, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,5, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,6, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,7, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,8, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,9, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xE,5, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xE,6, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xE,7, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xE,8, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xE,9, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xE,10, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xE,11, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xE,12, MD_PLN|MD_EZI|MD_ZI, FUNC3);

	scu_pinmux(2,9, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(2,10, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(2,11, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(2,12, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(2,13, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,0, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(1,1, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(1,2, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(2,8, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(2,7, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(2,6, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(2,2, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(2,1, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(2,0, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(6,8, MD_PLN|MD_EZI|MD_ZI, FUNC1);
	scu_pinmux(6,7, MD_PLN|MD_EZI|MD_ZI, FUNC1);
	scu_pinmux(0xD,16, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,15, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(14,0, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(14,1, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(14,2, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(14,3, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(14,4, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(0xA,4, MD_PLN|MD_EZI|MD_ZI, FUNC3);

  	/* Signals for static memory */
	scu_pinmux(1,6, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,3, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,5, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(1,4, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(6,3, MD_PLN|MD_EZI|MD_ZI, FUNC3);
	scu_pinmux(6,6, MD_PLN|MD_EZI|MD_ZI, FUNC1);
	scu_pinmux(0xD,12, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,13, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,11, MD_PLN|MD_EZI|MD_ZI, FUNC2);
	scu_pinmux(0xD,10, MD_PLN|MD_EZI|MD_ZI, FUNC2);

 	LPC_EMC->STATICWAITRD0 = 0x0000000E;
	LPC_EMC->CONTROL = 0x00000001;
	LPC_EMC->STATICCONFIG0 = 0x00000081; 
  	LPC_EMC->STATICWAITOEN0 = 0x00000001;
}
#endif 
