/*****************************************************************************
 * $Id:: main.c 6615 2011-02-25 14:28:13Z nxp16893                           $
 *
 * Project: FOC SCT Application Example for LPC1800	
 *
 * Description:
 *   FOC dual shunt using MCLIB
 *----------------------------------------------------------------------------
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/             
#include "app_setup.h"

/* Global variable declaration */
MC_STORAGE_Type mc_stored;					// Datastorage
volatile MC_DATA_Type mc;					// Definition of structure that holds all  
											// mc parameters for calculation											 
__align(4) volatile MC_COMMS_Type comms __attribute__((at(0x10084000))); 				
											// Definition of structure for comms
											// Located at upper half of 32kB SRAM block at 0x10084000
											// for UART->DMA transmission
volatile uint32_t systicks=0;				// Incremented in systick handler 

/* External variable declaration */
extern uint32_t SystemFrequency;

/* Function prototypes */
void Call_100ms(void);
void vDelay_Ms(uint32_t ms);
void GPIO0_IRQHandler(void);
void GPIO1_IRQHandler(void);
void GPIO2_IRQHandler(void);
void COMMS_Handler_Enable(uint8_t value);
void COMMS_Handler_SpeedEnable(uint8_t value);
void COMMS_Handler_SamplePoint(uint8_t ch, uint16_t value);

/* Main */
int main (void) 
{
	uint32_t cnt_100ms=0;
	uint8_t index;
	uint8_t sw3_pressed=FALSE;

	/* Disable FOC loop before enabling SCT */
	mc.ctrl.enabled = FALSE;						
	/* Configure all board specific peripherals */
	vAppInit(); 									
	printf("LPC18xxA/LPC43xxA FOC dual shunt.\n\r");

	/* Try to load stored parameters */
	if(!u8MS_DS_LoadParam(&mc_stored))
	{
		/* If no saved parameters are available then use default board settings */
		vAppSetDefault(&mc_stored);					
		
		/* Default MCLIB FOC parameters for storage */
		vMC_FOC_SetDefault(&mc_stored.var.foc);
		mc_stored.saved = FALSE;							
	}

	/* Import stored global paramaters */
	vMC_DS_ImportStorage(&mc_stored, (MC_DATA_Type *)&mc);				
	/* Init communication */

	/* Set Comms callback functions */
	comms.callbacks.Enable = COMMS_Handler_Enable;
	comms.callbacks.SpeedEnable = COMMS_Handler_SpeedEnable;
	comms.callbacks.SamplePoint = COMMS_Handler_SamplePoint;

	vMC_COMMS_Init((MC_COMMS_Type *)&comms, (MC_DATA_Type *)&mc, CONNECTION_TIMEOUT);
	/* Set default FOC parameters */
	vMC_FOC_Init((MC_DATA_Type *)&mc);								 		
	/* Start SCT */
	vSCT_Start();	
	
	printf("Initialisation complete.\n\rEnable motor with SW3 or GUI.\n\r");	

	while(1) 
    {
		/* Process received packets */
		if(u8MC_COMMS_GetRxPacketCnt() > 0)
		{
			index = u8MC_COMMS_GetNextPacketIndex();
			vMC_COMMS_ProcessPacket(index, (MC_DATA_Type *) &mc);	
		}
		
		/* Do timer loop */
		if(systicks>=cnt_100ms)		  	
		{
			cnt_100ms = systicks+100;
			Call_100ms();
		}
		
		if(SW3)				 
		{
			sw3_pressed = TRUE;
		}
		else if(sw3_pressed)
		{
			sw3_pressed = FALSE;
			if(mc.ctrl.enabled)
			{
				vMC_LIB_Disable((MC_DATA_Type*)&mc);
			}
			else
			{
				vMC_LIB_Enable((MC_DATA_Type*)&mc);			 	
			}
		}

	 	/* Check status lines */
		/* Should be handled with GPIO interrupts */
  		if(LPC_GPIO3->PIN & (1<<11))GPIO0_IRQHandler(); 	// Current trip interrupt 
  		if(LPC_GPIO6->PIN & (1<<24))GPIO1_IRQHandler(); 	// Brake stat0 interrupt, braking threshold reached
  		if(LPC_GPIO6->PIN & (1<<27))GPIO2_IRQHandler(); 	// Brake stat1 interrupt, braking forced by hardware 
	}
}

void COMMS_Handler_Enable(uint8_t value)
{
	if(value)
	{
		/* Enable control loop */
		if(mc.ctrl.curr_trip)vAppResetCurrTrip();
		/* Reset speed PID */
		vMC_LIB_PID_Reset((F6_10_PID_Type*)&mc.ctrl.pidSpeed);
		/* Enable control loop */
		vMC_LIB_Enable((MC_DATA_Type*)&mc);
	}
	else
	{
		/* Disable control loop */
		vMC_LIB_Disable((MC_DATA_Type*)&mc);
	}
}

void COMMS_Handler_SpeedEnable(uint8_t value)
{
	if(value)
	{
		/* Reset speed PID */
		vMC_LIB_PID_Reset((F6_10_PID_Type*)&mc.ctrl.pidSpeed);
	}
	mc.ctrl.speedEna = value;
}

void COMMS_Handler_SamplePoint(uint8_t ch, uint16_t value)
{
	mc.calib.IphADCpoint[ch] = value; 
	if(ch==0)
	{
		/* Set SCT ADC trigger */
		reload_adc_trig1(mc.calib.IphADCpoint[0]);
	}
}

/*****************************************************************************
** Function name:		Call_100ms
**
** Description:			100 ms Call function
**
** Parameters:			None
** Returned value:		None
**
*****************************************************************************/
void Call_100ms(void)
{
	static uint8_t i=0;

	vMC_COMMS_TimerFeed(100);
	if(!u8MC_COMMS_IsConnected())
	{
		vMC_COMMS_ConnReset();
	}

	/* Cycle leds */
	LPC_GPIO4->PIN &= ~0xF8;
	__NOP();
	__NOP();
	__NOP();
	LPC_GPIO4->PIN |= (1<<(3+i++));
	if(i>4)i=0;
}

/*----------------------------------------------------------------------------
  SysTick IRQ: Executed @ 100Hz
 *----------------------------------------------------------------------------*/
void SysTick_Handler (void) 			
{           
	systicks+=10; 
}

/*----------------------------------------------------------------------------
  vDelay_Ms
 *----------------------------------------------------------------------------*/
void vDelay_Ms(uint32_t ms)
{	
	systicks = 0;
	while(systicks<ms){__NOP();}
}

uint32_t regtestje;
/*----------------------------------------------------------------------------
  Current trip interrupt 
 *----------------------------------------------------------------------------*/
void GPIO0_IRQHandler(void)
{
	/* Register CURR_TRIP */
	mc.ctrl.curr_trip = TRUE;

	regtestje = LPC_GPIO_PIN_INT->IST;

	/* Disable motorcontrol */
	vMC_LIB_Disable((MC_DATA_Type*)&mc);
}

/*----------------------------------------------------------------------------
  Brake stat0 interrupt 
 *----------------------------------------------------------------------------*/
void GPIO1_IRQHandler(void)
{
	/* Braking threshold reached */
	regtestje = LPC_GPIO_PIN_INT->IST;
	mc.ctrl.brake_thresh = TRUE;
}

/*----------------------------------------------------------------------------
  Brake stat1 interrupt  
 *----------------------------------------------------------------------------*/
void GPIO2_IRQHandler(void)
{
	/* Braking forced by hardware */
	regtestje = LPC_GPIO_PIN_INT->IST;
	mc.ctrl.brake_forced = TRUE;

	/* Disable motorcontrol */
	vMC_LIB_Disable((MC_DATA_Type*)&mc);
}

/******************************************************************************
**                            End Of File
******************************************************************************/




