/*****************************************************************************
 *   adc.c:  ADC C file for NXP LPC17xx Family Microprocessors
 *
 *   Copyright(C) 2008, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2008.07.20  ver 1.00    Preliminary version, first Release
 *
*****************************************************************************/
#include "LPC18xx.h"			/* LPC18xx Peripheral Registers */
#include "type.h"
#include "lpc18xx_adc.h"
#include <stdio.h>

/* Private variables ---------------------------------------------------------*/
static ADC_CH_CALLBACK ADC_callbacks[2][8] =
{
	{
		NULL,  	/* ADC0 CH0 complete callback */
    	NULL,  	/* ADC0 CH1 complete callback */
		NULL,  	/* ADC0 CH2 complete callback */
    	NULL,  	/* ADC0 CH3 complete callback */
		NULL,  	/* ADC0 CH4 complete callback */
		NULL,  	/* ADC0 CH5 complete callback */
		NULL,  	/* ADC0 CH6 complete callback */
		NULL,  	/* ADC0 CH7 complete callback */
	},
	{
		NULL,  	/* ADC1 CH0 complete callback */
    	NULL,  	/* ADC1 CH1 complete callback */
		NULL,  	/* ADC1 CH2 complete callback */
    	NULL,  	/* ADC1 CH3 complete callback */
		NULL,  	/* ADC1 CH4 complete callback */
		NULL,  	/* ADC1 CH5 complete callback */
		NULL,  	/* ADC1 CH6 complete callback */
		NULL,  	/* ADC1 CH7 complete callback */
	},
};

/******************************************************************************
** Function name:		ADC0_IRQHandler
**
** Descriptions:		ADC0 interrupt handler
**
** Parameters:			None
** Returned value:		None
** 
******************************************************************************/
void ADC0_IRQHandler (void) 
{
	uint32_t stat;
	uint8_t i;
				
	stat = LPC_ADC0->STAT;				// Read ADC will clear the interrupt 
	
	for(i=0;i<8;i++)
	{
		if(stat & (ADC_ADINT|(1<<i)))	// Channel conversion complete
		{
			if(ADC_callbacks[0][i] != NULL)
			{
				(*ADC_callbacks[0][i])((LPC_ADC0->DR[i]>>6)& 0x3FF);	
			}
		}
	}	
 	return;
}


/******************************************************************************
** Function name:		ADC1_IRQHandler
**
** Descriptions:		ADC1 interrupt handler
**
** Parameters:			None
** Returned value:		None
** 
******************************************************************************/
void ADC1_IRQHandler (void) 
{
	uint32_t stat;
	uint8_t i;
				
	stat = LPC_ADC1->STAT;				// Read ADC will clear the interrupt 
	
	for(i=0;i<8;i++)
	{
		if(stat & (ADC_ADINT|(1<<i)))	// Channel conversion complete
		{
			if(ADC_callbacks[1][i] != NULL)
			{
				(*ADC_callbacks[1][i])((LPC_ADC1->DR[i]>>6)& 0x3FF);	
			}
		}
	}	
 	return;
}

/*****************************************************************************
** Function name:		ADC0Init
**
** Descriptions:		initialize ADC channel
**
** Parameters:			ADC clock rate
** Returned value:		None
** 
*****************************************************************************/
void vADC0Init(uint8_t channels, uint32_t clk,  uint8_t start, uint8_t edge)
{
	uint8_t i;
	
	// Using didicated ADC pins, no pinmux is nescesary
	LPC_ADC0->CR = 
	channels|  						/* ADC0 channel select */
	((VPB3Frequency/clk-1)<<8) | 	/* ((APB3Frequency/ADC_Clk-1)<<8) : ADC_Clk =  APB3Frequency / (CLKDIV + 1) */ 
	(0<<16) | 						/* BURST = 0, no BURST, software controlled */
	(0<<17) |  						/* CLKS = 0, 11 clocks/10 bits */
	(1<<21) |  						/* PDN = 1, normal operation */
	(start<<24) |  					/* START */
	(edge<<27);						/* EDGE */ 

	for(i=0;i<8;i++)ADC_callbacks[0][i]=NULL;
	return;
}

/*****************************************************************************
** Function name:		ADC1Init
**
** Descriptions:		initialize ADC channel
**
** Parameters:			ADC clock rate
** Returned value:		None
** 
*****************************************************************************/
void vADC1Init(uint8_t channels, uint32_t clk,  uint8_t start, uint8_t edge)
{
	uint8_t i;
	
	// Using didicated ADC pins, no pinmux is nescesary
	LPC_ADC1->CR = 
	channels|  						/* ADC1 channel select */
	((VPB3Frequency/clk-1)<<8) | 	/* ((APB3Frequency/ADC_Clk-1)<<8) : ADC_Clk =  APB3Frequency / (CLKDIV + 1) */ 
	(0<<16) | 						/* BURST = 0, no BURST, software controlled */
	(0<<17) |  						/* CLKS = 0, 11 clocks/10 bits */
	(1<<21) |  						/* PDN = 1, normal operation */
	(start<<24) |  					/* START */
	(edge<<27);						/* EDGE */ 

	for(i=0;i<8;i++)ADC_callbacks[1][i]=NULL;
	return;
}


/*****************************************************************************
** Function name:		vADC_SetChCallback
**
** Descriptions:		set ADC channelconversion complete callbacks
**
** Parameters:			ADC port, channel number and callback function
** Returned value:		None
** 
*****************************************************************************/
void vADC_SetChCallback(uint8_t port, uint8_t ch, ADC_CH_CALLBACK callback)
{
	ADC_callbacks[port][ch]= callback;

	switch(port)
	{
		case(0):	// ADC 0 interrupt
			NVIC_EnableIRQ(ADC0_IRQn);
			LPC_ADC0->INTEN = (1<<ch);			/* Enable channel interrupt */			
			break;
		case(1):	// ADC 1 interrupt
			NVIC_EnableIRQ(ADC1_IRQn);
			LPC_ADC1->INTEN = (1<<ch);			/* Enable channel interrupt */			
			break;
		default:
			break;
	}
}

/*****************************************************************************
** Function name:		u16ADC_ReadDR
**
** Descriptions:		read ADC conversion results
**
** Parameters:			ADC port, channel number 
** Returned value:		adc value
** 
*****************************************************************************/
uint16_t u16ADC_ReadDR(uint8_t port, uint8_t ch)
{
 	switch(port)
	{
		case(0):	// ADC 0 
			return (LPC_ADC0->DR[ch]>>6) & 0x3FF;	
		case(1):	// ADC 1 
			return (LPC_ADC1->DR[ch]>>6) & 0x3FF;	
		default:
			return 0;
	}
}	
		
/*********************************************************************************
**                            End Of File
*********************************************************************************/
