/*****************************************************************************
 *   qei.c:  QEI module file for NXP LPC18xx Family Microprocessors
 *
 *   Copyright(C) 2010, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2010.04.22  ver 1.00    Preliminary version, first Release
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

#include "LPC18xx.h"                    		/* LPC18xx definitions                */
#include "type.h"
#include "lpc18xx_scu.h"
#include "lpc18xx_qei.h"

/*****************************************************************************
** Function name:		vQEI_Init
**
** Description:			Initialize QEI
**
*****************************************************************************/
void vQEI_Init(void)
{
   	
	scu_pinmux(0xA ,1 , MD_PDN|MD_EZI, FUNC1); 	// QEI0 IDX, Digital input with pull-up	  CON_ES_22=BGA_AB14=PA_1
	scu_pinmux(0xA ,2 , MD_PDN|MD_EZI, FUNC1); 	// QEI0 PHB, Digital input with pull-up	  CON_ES_23=BGA_AG4 =PA_2
	scu_pinmux(0xA ,3 , MD_PDN|MD_EZI, FUNC1); 	// QEI0 PHA, Digital input with pull-up	  CON_ES_24=BGA_AC10=PA_3

	LPC_QEI->LOAD = QEI_LOAD; 					// Velocity timer match value
	LPC_QEI->CON  = 0x0F;             			// reset index counter, velocity, reset on index
	LPC_QEI->CONF = QEI_CAPMODE | 				// BOTH PhA and PhB edges are counted (4X) 
					/* QEI_DIRINV | */ 			// Direction invert.
					QEI_CRESPI |				// Continuously reset position counter on index
					/*QEI_INVINX |*/ 			// Invert Index
					QEI_INXGATE(0xF);			// Pass all index 
    LPC_QEI->MAXPOS = QEI_MAXPOS;         		// set QEI max pos
	LPC_QEI->FILTERPHA = 200;					// set number of samples for valid value
	LPC_QEI->FILTERPHB = 200;					// set number of samples for valid value
	LPC_QEI->FILTERINX = 200;					// set number of samples for valid value
    LPC_QEI->IEC  = 0x000FFFF;        			// disable all QEI interrupts
    LPC_QEI->CLR  = 0x000FFFF;        			// clear all QEI pending interrupts   
	LPC_QEI->IES  = 0;//ENCLK_Int | INX_Int;	
	return;     
}

/*****************************************************************************
** Function name:		vQEI_SetDirection
**
** Descriptions:		Set QEI direction
**						0 = not inverted
**						1 = inverted
**
*****************************************************************************/
void vQEI_SetDirection(uint8_t dir)
{
	if(dir)LPC_QEI->CONF |= QEI_DIRINV;
	else LPC_QEI->CONF &= ~QEI_DIRINV;		
}

/*********************************************************************//**
 * @brief		Enable/Disable specified interrupt in QEI peripheral
 * @param[in]	ulIntType		Interrupt Flag Status type, should be:
 * 					- QEI_INTFLAG_INX_Int		: index pulse was detected interrupt
 * 					- QEI_INTFLAG_TIM_Int		: Velocity timer over flow interrupt
 * 					- QEI_INTFLAG_VELC_Int		: Capture velocity is less than compare interrupt
 * 					- QEI_INTFLAG_DIR_Int		: Change of direction interrupt
 * 					- QEI_INTFLAG_ERR_Int		: An encoder phase error interrupt
 * 					- QEI_INTFLAG_ENCLK_Int		: An encoder clock pulse was detected interrupt
 * 					- QEI_INTFLAG_POS0_Int		: position 0 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_POS1_Int		: position 1 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_POS2_Int		: position 2 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_REV_Int		: Index compare value is equal to the current index count interrupt
 * 					- QEI_INTFLAG_POS0REV_Int	: Combined position 0 and revolution count interrupt
 * 					- QEI_INTFLAG_POS1REV_Int	: Combined position 1 and revolution count interrupt
 * 					- QEI_INTFLAG_POS2REV_Int	: Combined position 2 and revolution count interrupt
 * @param[in]	NewState	New function state, should be:
 *					- DISABLE
 *					- ENABLE
 * @return		None
 **********************************************************************/
void vQEI_IntCmd(uint32_t ulIntType, FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		LPC_QEI->IES = ulIntType;
	}
	else
	{
		LPC_QEI->IEC = ulIntType;
	}
}

/*********************************************************************//**
 * @brief		Clear (force) specified interrupt (pending) in QEI peripheral
 * @param[in]	ulIntType		Interrupt Flag Status type, should be:
 * 					- QEI_INTFLAG_INX_Int		: index pulse was detected interrupt
 * 					- QEI_INTFLAG_TIM_Int		: Velocity timer over flow interrupt
 * 					- QEI_INTFLAG_VELC_Int		: Capture velocity is less than compare interrupt
 * 					- QEI_INTFLAG_DIR_Int		: Change of direction interrupt
 * 					- QEI_INTFLAG_ERR_Int		: An encoder phase error interrupt
 * 					- QEI_INTFLAG_ENCLK_Int		: An encoder clock pulse was detected interrupt
 * 					- QEI_INTFLAG_POS0_Int		: position 0 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_POS1_Int		: position 1 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_POS2_Int		: position 2 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_REV_Int		: Index compare value is equal to the current index count interrupt
 * 					- QEI_INTFLAG_POS0REV_Int	: Combined position 0 and revolution count interrupt
 * 					- QEI_INTFLAG_POS1REV_Int	: Combined position 1 and revolution count interrupt
 * 					- QEI_INTFLAG_POS2REV_Int	: Combined position 2 and revolution count interrupt
 * @return		None
 **********************************************************************/
void vQEI_IntClear(uint32_t ulIntType)
{
	LPC_QEI->CLR = ulIntType;
}

/*********************************************************************//**
 * @brief		Check whether if specified interrupt flag status in QEI
 * 				peripheral is set or not
 * @param[in]	ulIntType Interrupt Flag Status type, should be:
 * 					- QEI_INTFLAG_INX_Int		: index pulse was detected interrupt
 * 					- QEI_INTFLAG_TIM_Int		: Velocity timer over flow interrupt
 * 					- QEI_INTFLAG_VELC_Int		: Capture velocity is less than compare interrupt
 * 					- QEI_INTFLAG_DIR_Int		: Change of direction interrupt
 * 					- QEI_INTFLAG_ERR_Int		: An encoder phase error interrupt
 * 					- QEI_INTFLAG_ENCLK_Int		: An encoder clock pulse was detected interrupt
 * 					- QEI_INTFLAG_POS0_Int		: position 0 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_POS1_Int		: position 1 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_POS2_Int		: position 2 compare value is equal to the current position interrupt
 * 					- QEI_INTFLAG_REV_Int		: Index compare value is equal to the current index count interrupt
 * 					- QEI_INTFLAG_POS0REV_Int	: Combined position 0 and revolution count interrupt
 * 					- QEI_INTFLAG_POS1REV_Int	: Combined position 1 and revolution count interrupt
 * 					- QEI_INTFLAG_POS2REV_Int	: Combined position 2 and revolution count interrupt
 * @return		New State of specified interrupt flag status (SET or RESET)
 **********************************************************************/
FlagStatus fsQEI_GetIntStatus(uint32_t ulIntType)
{
	return((LPC_QEI->INTSTAT & ulIntType) ? SET : RESET);
}



			




  




						 

