/*****************************************************************************
 *   qei.h:  Header file for NXP LPC18xx Family Microprocessors
 *
 *   Copyright(C) 2010, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2010.04.22  ver 1.00    Preliminary version, first Release
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

#ifndef __QEI_H
#define __QEI_H

#include "mc_inc.h"

/** QEI interrupt flag type */
#define QEI_INTFLAG_INX_Int			((uint32_t)(1<<0))	/**< index pulse was detected interrupt */
#define QEI_INTFLAG_TIM_Int			((uint32_t)(1<<1))	/**< Velocity timer over flow interrupt */
#define QEI_INTFLAG_VELC_Int		((uint32_t)(1<<2))	/**< Capture velocity is less than compare interrupt */
#define QEI_INTFLAG_DIR_Int			((uint32_t)(1<<3))	/**< Change of direction interrupt */
#define QEI_INTFLAG_ERR_Int			((uint32_t)(1<<4))	/**< An encoder phase error interrupt */
#define QEI_INTFLAG_ENCLK_Int		((uint32_t)(1<<5))	/**< An encoder clock pulse was detected interrupt */
#define QEI_INTFLAG_POS0_Int		((uint32_t)(1<<6))	/**< position 0 compare value is equal to the
														current position interrupt */
#define QEI_INTFLAG_POS1_Int		((uint32_t)(1<<7))	/**< position 1 compare value is equal to the
														current position interrupt */
#define QEI_INTFLAG_POS2_Int		((uint32_t)(1<<8))	/**< position 2 compare value is equal to the
														current position interrupt */
#define QEI_INTFLAG_REV_Int			((uint32_t)(1<<9))	/**< Index compare value is equal to the current
														index count interrupt */
#define QEI_INTFLAG_POS0REV_Int		((uint32_t)(1<<10))	/**< Combined position 0 and revolution count interrupt */
#define QEI_INTFLAG_POS1REV_Int		((uint32_t)(1<<11))	/**< Combined position 1 and revolution count interrupt */
#define QEI_INTFLAG_POS2REV_Int		((uint32_t)(1<<12))	/**< Combined position 2 and revolution count interrupt */

#define QEI_MAXPOS	((QEI_COUNTS_PER_TURN * 4)-1)	// With CAPMODE=1, CPT x 4
#define QEI_CLK		120000000					// [Hz] peripheral clockspeed
#define QEI_RPM_RESOLUTION	1

// The original formula for QEI RPM is: 
// QEI_RPM = (QEI_CLK * LPC_QEI->CAP * 60) / (QEI_LOAD * (QEI_CNTS+1) * 4)
// QEI_LOAD can now be calculated with the needed resoluation
// QEI_LOAD = Match value for velocity timer
// A velocity capture of 1 has to correspond to 'QEI_RPM_RESOLUTION' 
#define QEI_RPM (LPC_QEI->CAP*QEI_RPM_RESOLUTION)

// QEI_RPM_RESOLUTION = (QEI_CLK * CAP=1 * 60) / (QEI_LOAD * QEI_CNTS * 4)
// Rewrite for QEI_LOAD:
#define QEI_LOAD (QEI_CLK * 15ul) / (QEI_RPM_RESOLUTION * QEI_COUNTS_PER_TURN)	 

// At QEI_CLK=120MHz the QEI PID frequency becomes:
// QEISPEEDLOOPFREQ = QEI_CLK / QEI_LOAD = 120 MHz/ 3600000 = 55.85 Hz
// res: 
// QEI_RPM_RESOLUTION = (QEI_CLK * CAP=1 * 60) / (QEI_LOAD * QEI_CNTS * 4)
// 120000000 * 1* 60 / ( 2148591 * 500 * 4) = 1,6755160819145563938467431377491


// With QEI_RPM_RESOLUTION = 1 and QEI_CNTS = 500 the LOAD becomes:
// QEI_LOAD = (QEI_CLK * 60) / (1 * QEI_CNTS * 4) = 3600000
// At QEI_CLK=120MHz the QEI PID frequency becomes:
// QEISPEEDLOOPFREQ = QEI_CLK / QEI_LOAD = 120 MHz/ 3600000 = 33 Hz


#define QEI_INXGATE(N)	(N<<16)	// Index gating configuration.		
#define QEI_CRESPI	(1<<4)		// Continuously reset position counter on index
#define QEI_INVINX 	(1<<3)		// Invert Index. When set, inverts the sense of the index input. 
#define QEI_CAPMODE	(1<<2)		// Capture Mode. When = 0, only PhA edges are counted (2X). When = 1, 
								// BOTH PhA and PhB edges are counted (4X), increasing resolution but decreasing range.
#define QEI_SIGMODE (1<<1)		// Signal Mode. When = 0, PhA and PhB function as quadrature encoder inputs. When = 1, 
								// PhA functions as the direction signal and PhB functions as the clock signal.
#define QEI_DIRINV 	(1<<0)		// Direction invert. When = 1, complements the DIR bit. 

// clock is the controller clock rate
// ppr is the number of pulses per revolution of the physical encoder
// edges is 2 or 4, based on the capture mode set in the QEICON register (2 for
// CapMode set to 0 and 4 for CapMode set to 1)

void vQEI_Init(void);
void vQEI_SetDirection(uint8_t dir);
void vQEI_IntCmd(uint32_t ulIntType, FunctionalState NewState);
void vQEI_IntClear(uint32_t ulIntType);
FlagStatus fsQEI_GetIntStatus(uint32_t ulIntType);

/*****************************************************************************
** Function name:		u32QEI_Position
**
** Descriptions:		Return QEI position 
**
*****************************************************************************/
static __inline uint32_t u32QEI_Position(void)
{
	return LPC_QEI->POS;
}

/*****************************************************************************
** Function name:		u8QEI_Direction
**
** Descriptions:		Set QEI direction
**						0 = not inverted
**						1 = inverted
**
*****************************************************************************/
static __inline uint8_t u8QEI_Direction(void)
{
	return LPC_QEI->STAT & (1<<0);
}

/*****************************************************************************
** Function name:		u32QEI_Index
**
** Descriptions:		Set QEI direction
**						0 = not inverted
**						1 = inverted
**
*****************************************************************************/
static __inline uint32_t u32QEI_Index(void)
{
	return LPC_QEI->INXCNT; 
}

/*****************************************************************************
** Function name:		vQEI_ResInx
**
** Descriptions:		Reset index counter 
**
*****************************************************************************/
static __inline void vQEI_ResInx(void)
{
	LPC_QEI->CON |= (1<<3); 		// RESI; reset index counter
	while(LPC_QEI->INXCNT != 0);	// Wait for index counter to become 0
}

/*****************************************************************************
** Function name:		vQEI_ResPos
**
** Descriptions:		Reset position counter 
**
*****************************************************************************/
static __inline void vQEI_ResPos(void)
{
	LPC_QEI->CON |= (1<<0); 		// RESP; reset position counter
}

/*****************************************************************************
** Function name:		u8QEI_IsVelocityUpdated
**
** Descriptions:		returns 1 if velocity interrupt occured
**						
*****************************************************************************/
static __inline uint8_t u8QEI_IsVelocityUpdated(void)
{
	if(LPC_QEI->INTSTAT&(1<<1))	// bit1 : TIM_int, velocity timer overflow
	{
		LPC_QEI->CLR = (1<<1);	// Clear bit 1 : TIM_int, velocity timer overflow
		return 1;
	}
	return 0;
}

/*****************************************************************************
** Function name:		int32_t i32QEI_RPM
**
** Descriptions:		returns rpm 
**						
*****************************************************************************/
static __inline int32_t i32QEI_RPM(void)
{
	if(u8QEI_Direction())
	{
		return -QEI_RPM; 	
	}
	else
	{
		return QEI_RPM; 		
	}
}				

#endif
