/*****************************************************************************
 *   uart.c:  UART API file for NXP LPC18xx Family Microprocessors
 *
 *   Copyright(C) 2010, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2010.04.20  ver 1.00    Preliminary version, first Release
 *
******************************************************************************/
#include "LPC18xx.h"
#include "type.h"
#include "lpc18xx_uart.h"
#include "lpc18xx_scu.h"

/* Private variables ---------------------------------------------------------*/
static UART_RX_CALLBACK UART_rx_callbacks[4] =
{
	NULL,  	/* UART0_RX_CALLBACK */
    NULL,   /* UART1_RX_CALLBACK */
	NULL,  	/* UART2_RX_CALLBACK */
    NULL,   /* UART3_RX_CALLBACK */       
};

LinkedList_Type ll[24];

volatile uint32_t UART0Status, UART1Status, UART2Status, UART3Status;
volatile uint8_t UART0TxEmpty = 1, UART1TxEmpty = 1, UART2TxEmpty = 1, UART3TxEmpty = 1;

/*****************************************************************************
** Function name:		u32UARTInit
**
** Descriptions:		Initialize UART port, setup pin select,
**						clock, parity, stop bits, FIFO, etc.
**
** Parameters:			UART_Init_Type containing portNum, baudrate and 
**						UART_RX_CALLBACK
** Returned value:		true or false, return false only if the 
**						interrupt handler can't be installed to the 
**						VIC table
** 
*****************************************************************************/
uint32_t u32UARTInit( UART_Init_Type* pInit)
{
  	uint32_t Fdiv;

	UART_rx_callbacks[pInit->portNum]= pInit->rx_callback;
 		
	if(pInit->portNum == 0)
	{
		scu_pinmux(0xF,10,MD_PDN,FUNC1);    	/* PF.10, U0_TXD */
		scu_pinmux(0xF,11,MD_PDN|MD_EZI,FUNC1);	/* PF.11, U0_RXD */
		
		LPC_USART0->LCR = 0x83;					/* 8 bits, no Parity, 1 Stop bit */
		Fdiv = ( UART0Frequency / 16 ) / pInit->baudrate ;	/*baud rate */
		LPC_USART0->DLM = Fdiv / 256;							
		LPC_USART0->DLL = Fdiv % 256;
		LPC_USART0->LCR = 0x03;					/* DLAB = 0 */
		LPC_USART0->FCR = 0xF;					/* Enable and reset TX and RX FIFO and DMA mode. */
		
		if(pInit->rx_callback != NULL)
		{
			NVIC_EnableIRQ(USART0_IRQn);
			LPC_USART0->IER = IER_RBR | IER_THRE | IER_RLS;	/* Enable UART0 interrupt */
		}
		return (TRUE);
	}
	else if(pInit->portNum == 1)
  	{
		scu_pinmux(0xC,13,MD_PDN,FUNC2);    	/* PC.13, U1_TXD */
		scu_pinmux(0xC,14,MD_PDN|MD_EZI,FUNC2);	/* PC.14, U1_RXD */
		
	    LPC_UART1->LCR = 0x83;					/* 8 bits, no Parity, 1 Stop bit */
		Fdiv = ( UART1Frequency / 16 ) / pInit->baudrate ;	/*baud rate */
	    LPC_UART1->DLM = Fdiv / 256;							
	    LPC_UART1->DLL = Fdiv % 256;
		LPC_UART1->LCR = 0x03;					/* DLAB = 0 */
	    LPC_UART1->FCR = 0x07;					/* Enable and reset TX and RX FIFO. */
	
	   	if(pInit->rx_callback != NULL)
		{
			NVIC_EnableIRQ(UART1_IRQn);
			LPC_UART1->IER = IER_RBR | IER_THRE | IER_RLS;	/* Enable UART1 interrupt */
		}
	    return (TRUE);
	}
	else if(pInit->portNum == 2)
	{
		scu_pinmux(0x1,15,MD_PDN,FUNC1);    	/* P1.15, U2_TXD */
		scu_pinmux(0x1,16,MD_PDN|MD_EZI,FUNC1); /* P1.16, U2_RXD */
		
	    LPC_USART2->LCR = 0x83;					/* 8 bits, no Parity, 1 Stop bit */
		Fdiv = ( UART2Frequency / 16 ) / pInit->baudrate ;	/*baud rate */
	    LPC_USART2->DLM = Fdiv / 256;							
	    LPC_USART2->DLL = Fdiv % 256;
		LPC_USART2->LCR = 0x03;					/* DLAB = 0 */
	    LPC_USART2->FCR = 0x07;					/* Enable and reset TX and RX FIFO. */
	
	   	if(pInit->rx_callback != NULL)
		{
		   	NVIC_EnableIRQ(USART2_IRQn);
		    LPC_USART2->IER = IER_RBR | IER_THRE | IER_RLS;	/* Enable UART2 interrupt */
		}
    	return (TRUE);
	}		
	else if(pInit->portNum == 3)
	{
		scu_pinmux(0xF,2,MD_PDN,FUNC1);    		/* PF.2, U3_TXD */
		scu_pinmux(0xF,3,MD_PDN|MD_EZI,FUNC1);  /* PF.3, U3_RXD */
		
	    LPC_USART3->LCR = 0x83;					/* 8 bits, no Parity, 1 Stop bit */
		Fdiv = ( UART3Frequency / 16 ) / pInit->baudrate ;	/*baud rate */
	    LPC_USART3->DLM = Fdiv / 256;							
	    LPC_USART3->DLL = Fdiv % 256;
		LPC_USART3->LCR = 0x03;					/* DLAB = 0 */
	    LPC_USART3->FCR = 0x07;					/* Enable and reset TX and RX FIFO. */
	
	   	if(pInit->rx_callback != NULL)
		{
			NVIC_EnableIRQ(USART3_IRQn);
		    LPC_USART3->IER = IER_RBR | IER_THRE | IER_RLS;	/* Enable UART3 interrupt */
		}
	    return (TRUE);
	}
	return( FALSE ); 
}

/*****************************************************************************
** Function name:		UART0_IRQHandler
**
** Descriptions:		UART0 interrupt handler
**
** Parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void UART0_IRQHandler (void) 
{
  uint8_t IIRValue, LSRValue;
  uint8_t Dummy = Dummy;
	
  IIRValue = LPC_USART0->IIR;
    
  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */
  if ( IIRValue == IIR_RLS )		/* Receive Line Status */
  {
	LSRValue = LPC_USART0->LSR;
	/* Receive Line Status */
	if ( LSRValue & (LSR_OE|LSR_PE|LSR_FE|LSR_RXFE|LSR_BI) )
	{
	  /* There are errors or break interrupt */
	  /* Read LSR will clear the interrupt */
	  UART0Status = LSRValue;
	  Dummy = LPC_USART0->RBR;		/* Dummy read on RX to clear 
							interrupt, then bail out */
	  return;
	}
	if ( LSRValue & LSR_RDR )	/* Receive Data Ready */			
	{
	  /* If no error on RLS, normal ready, save into the data buffer. */
	  /* Note: read RBR will clear the interrupt */

		if (UART_rx_callbacks[0] != NULL)
		{
    		(*UART_rx_callbacks[0])(LPC_USART0->RBR);
    	}
	}
  }
  else if ( IIRValue == IIR_RDA )	/* Receive Data Available */
  {
		if (UART_rx_callbacks[0] != NULL)
		{
			(*UART_rx_callbacks[0])(LPC_USART0->RBR);
		}	 
  }
  else if ( IIRValue == IIR_CTI )	/* Character timeout indicator */
  {
	/* Character timeout indicator */
	/* The mini. RX trigger level is 16, so, whenever RX FIFO is
	not empty, this CTI will be set after 4 character time, simply 
	read data from the FIFO when this interrupt occurs. */
		if (UART_rx_callbacks[0] != NULL)
		{
			(*UART_rx_callbacks[0])(LPC_USART0->RBR);
		}
  }
  else if ( IIRValue == IIR_THRE )	/* THRE, transmit holding register empty */
  {
	/* THRE interrupt */
	LSRValue = LPC_USART0->LSR;		/* Check status in the LSR to see if
									valid data in U0THR or not */
	if ( LSRValue & LSR_THRE )
	{
	  UART0TxEmpty = 1;
	}
	else
	{
	  UART0TxEmpty = 0;
	}
  }
  return;    
}

/*****************************************************************************
** Function name:		UART1_IRQHandler
**
** Descriptions:		UART1 interrupt handler
**
** Parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void UART1_IRQHandler (void) 
{
  uint8_t IIRValue, LSRValue;
  uint8_t Dummy = Dummy;
	
  IIRValue = LPC_UART1->IIR;
    
  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */
  if ( IIRValue == IIR_RLS )		/* Receive Line Status */
  {
	LSRValue = LPC_UART1->LSR;
	/* Receive Line Status */
	if ( LSRValue & (LSR_OE|LSR_PE|LSR_FE|LSR_RXFE|LSR_BI) )
	{
	  /* There are errors or break interrupt */
	  /* Read LSR will clear the interrupt */
	  UART1Status = LSRValue;
	  Dummy = LPC_UART1->RBR;		/* Dummy read on RX to clear 
								interrupt, then bail out */
	  return;
	}
	if ( LSRValue & LSR_RDR )	/* Receive Data Ready */			
	{
	  /* If no error on RLS, normal ready, save into the data buffer. */
	  /* Note: read RBR will clear the interrupt */
	  	if (UART_rx_callbacks[1] != NULL)
		{
			(*UART_rx_callbacks[1])(LPC_UART1->RBR);
		}
	}
  }
  else if ( IIRValue == IIR_RDA )	/* Receive Data Available */
  {
		/* Receive Data Available */
		if (UART_rx_callbacks[1] != NULL)
		{
		(*UART_rx_callbacks[1])(LPC_UART1->RBR);
		}
  }
  else if ( IIRValue == IIR_CTI )	/* Character timeout indicator */
  {
	/* Character Time-out indicator */
	UART1Status |= 0x100;		/* Bit 9 as the CTI error */
  }
  else if ( IIRValue == IIR_THRE )	/* THRE, transmit holding register empty */
  {
	/* THRE interrupt */
	LSRValue = LPC_UART1->LSR;		/* Check status in the LSR to see if
								valid data in U0THR or not */
	if ( LSRValue & LSR_THRE )
	{
	  UART1TxEmpty = 1;
	}
	else
	{
	  UART1TxEmpty = 0;
	}
  }
  return;
}

/*****************************************************************************
** Function name:		UART2_IRQHandler
**
** Descriptions:		UART2 interrupt handler
**
** Parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void UART2_IRQHandler (void) 
{
  uint8_t IIRValue, LSRValue;
  uint8_t Dummy = Dummy;
	
  IIRValue = LPC_USART2->IIR;
    
  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */
  if ( IIRValue == IIR_RLS )		/* Receive Line Status */
  {
	LSRValue = LPC_USART2->LSR;
	/* Receive Line Status */
	if ( LSRValue & (LSR_OE|LSR_PE|LSR_FE|LSR_RXFE|LSR_BI) )
	{
	  /* There are errors or break interrupt */
	  /* Read LSR will clear the interrupt */
	  UART2Status = LSRValue;
	  Dummy = LPC_USART2->RBR;		/* Dummy read on RX to clear 
							interrupt, then bail out */
	  return;
	}
	if ( LSRValue & LSR_RDR )	/* Receive Data Ready */			
	{
	  	if (UART_rx_callbacks[2] != NULL)
		{
			(*UART_rx_callbacks[2])(LPC_USART2->RBR);
		}
	}
  }
  else if ( IIRValue == IIR_RDA )	/* Receive Data Available */
  {
		/* Receive Data Available */
		if (UART_rx_callbacks[2] != NULL)
		{
			(*UART_rx_callbacks[2])(LPC_USART2->RBR);
		}
  }
  else if ( IIRValue == IIR_CTI )	/* Character timeout indicator */
  {
	/* Character Time-out indicator */
	UART2Status |= 0x100;		/* Bit 9 as the CTI error */
  }
  else if ( IIRValue == IIR_THRE )	/* THRE, transmit holding register empty */
  {
	/* THRE interrupt */
	LSRValue = LPC_USART2->LSR;		/* Check status in the LSR to see if
									valid data in U0THR or not */
	if ( LSRValue & LSR_THRE )
	{
	  UART2TxEmpty = 1;
	}
	else
	{
	  UART2TxEmpty = 0;
	}
  }
  return;    
}

/*****************************************************************************
** Function name:		UART3_IRQHandler
**
** Descriptions:		UART3 interrupt handler
**
** Parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void UART3_IRQHandler (void) 
{
  uint8_t IIRValue, LSRValue;
  uint8_t Dummy = Dummy;
	
  IIRValue = LPC_USART3->IIR;
    
  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */
  if ( IIRValue == IIR_RLS )		/* Receive Line Status */
  {
	LSRValue = LPC_USART3->LSR;
	/* Receive Line Status */
	if ( LSRValue & (LSR_OE|LSR_PE|LSR_FE|LSR_RXFE|LSR_BI) )
	{
	  /* There are errors or break interrupt */
	  /* Read LSR will clear the interrupt */
	  UART3Status = LSRValue;
	  Dummy = LPC_USART3->RBR;		/* Dummy read on RX to clear 
							interrupt, then bail out */
	  return;
	}
	if ( LSRValue & LSR_RDR )	/* Receive Data Ready */			
	{
	  	if (UART_rx_callbacks[3] != NULL)
		{
			(*UART_rx_callbacks[3])(LPC_USART3->RBR);
		}
	}
  }
  else if ( IIRValue == IIR_RDA )	/* Receive Data Available */
  {
		/* Receive Data Available */
	  	if (UART_rx_callbacks[3] != NULL)
		{
			(*UART_rx_callbacks[3])(LPC_USART3->RBR);
		}
  }
  else if ( IIRValue == IIR_CTI )	/* Character timeout indicator */
  {
	/* Character Time-out indicator */
	UART3Status |= 0x100;		/* Bit 9 as the CTI error */
  }
  else if ( IIRValue == IIR_THRE )	/* THRE, transmit holding register empty */
  {
	/* THRE interrupt */
	LSRValue = LPC_USART3->LSR;		/* Check status in the LSR to see if
									valid data in U0THR or not */
	if ( LSRValue & LSR_THRE )
	{
	  UART3TxEmpty = 1;
	}
	else
	{
	  UART3TxEmpty = 0;
	}
  }
  return;    
}

/*****************************************************************************
** Function name:		UARTSend
**
** Descriptions:		Send a block of data to the UART 0 port based
**						on the data length
**
** Parameters:			portNum, buffer pointer, and data length
** Returned value:		None
** 
*****************************************************************************/
void vUARTSend( uint32_t portNum, uint8_t *BufferPtr, uint32_t Length )
{
  if ( portNum == 0 )
  {
    while ( Length != 0 )
    {
	  /* THRE status, contain valid data */
	  while ( !(UART0TxEmpty & 0x01) );	
	  LPC_USART0->THR = *BufferPtr;
	  UART0TxEmpty = 0;	/* not empty in the THR until it shifts out */
	  BufferPtr++;
	  Length--;
	}
  }
  else if ( portNum == 1 )
  {
	while ( Length != 0 )
    {
	  /* THRE status, contain valid data */
	  while ( !(UART1TxEmpty & 0x01) );	
	  LPC_UART1->THR = *BufferPtr;
	  UART1TxEmpty = 0;	/* not empty in the THR until it shifts out */
	  BufferPtr++;
	  Length--;
    }
  }
  else if ( portNum == 2 )
  {
	while ( Length != 0 )
    {
	  /* THRE status, contain valid data */
	  while ( !(UART2TxEmpty & 0x01) );	
	  LPC_USART2->THR = *BufferPtr;
	  UART2TxEmpty = 0;	/* not empty in the THR until it shifts out */
	  BufferPtr++;
	  Length--;
    }
  }
  else if ( portNum == 3 )
  {
	while ( Length != 0 )
    {
	  /* THRE status, contain valid data */
	  while ( !(UART3TxEmpty & 0x01) );	
	  LPC_USART3->THR = *BufferPtr;
	  UART3TxEmpty = 0;	/* not empty in the THR until it shifts out */
	  BufferPtr++;
	  Length--;
    }
  }
  return;
}

void vUARTPutc(uint8_t port, uint8_t c)
{
	switch(port)
	{
		case(0):
			while(!(LPC_USART0->LSR&LSR_THRE));	// Wait for Tx to complete
			LPC_USART0->THR = c;				   	// New tx char
			break;
		case(1):
			while(!(LPC_UART1->LSR&LSR_THRE));	// Wait for Tx to complete
			LPC_UART1->THR = c;				   	// New tx char
			break;
		case(2):
			while(!(LPC_USART2->LSR&LSR_THRE));	// Wait for Tx to complete
			LPC_USART2->THR = c;				   	// New tx char
			break;
		case(3):
			while(!(LPC_USART3->LSR&LSR_THRE));	// Wait for Tx to complete
			LPC_USART3->THR = c;				   	// New tx char
			break;
		default:
			break;
	}
}

/******************************************************************************
** Function name:		DMA_UART_Send
**
** Descriptions:		
**
** Parameters:			
** Returned value:		
** 
******************************************************************************/
void vUART0_DMA_Send(uint32_t *data, uint32_t size)
{
	uint32_t i=0;

	// Build LinkedList array
	for(i=0;i<(size>>11);i++)									// 2>>11=2048=Max DMA TransferSize
	{														
	    ll[i].SRC  = (unsigned long)data+(i<<11);	
		ll[i].DST  = (uint32_t) &(LPC_USART0->THR); 			// UART0 THR
		ll[i].LLI  = (((uint32_t)&ll[i+1]) &0xFFFFFFFC) | 1;	// Next linked list, AHB master 1
//		ll[i].CTRL = (2048<<0) | (0x00 << 12) | (0x3 << 15) |
//    	             (0x00 << 18) | (0x00 << 21) | (1 << 26) | 
//                     (0 << 27) | (0<<31);						// No terminal count interrupt

		ll[i].CTRL	= (	
					2048<<0 |			// Transfer size 
					0x0 << 12 |			// Source burst size 1
					0x0 << 15 |			// Dest burst size 1
					0x0 << 18 |			// 8bit source width
					0x0 << 21 | 		// 8bit dest width UART
					0x1 << 24 |			// Use AHB Master 1 for source transfer
					0x1 << 25 |			// Use AHB Master 1 for dest transfer
					0x1 << 26 |			// Enable source increment
					0x0 << 27 |			// Disable dest increment
					0x1 << 28 |			// Privileged mode 
					0x1 << 29 |			// Bufferable 
					0x0 << 31 			// Terminal count interrupt disabled.
					);

	}
	ll[i].SRC  = (unsigned long)data+(i<<11);	
	ll[i].DST  = (uint32_t) &(LPC_USART0->THR); 				// UART0 THR
	ll[i].LLI  = 0;												// Last linked list is end of list
//	ll[i].CTRL = ((size%2048)<<0) | (0x00 << 12) | (0x3 << 15) |
//    	         (0x00 << 18) | (0x00 << 21) | (1 << 26) | 
//                 (0 << 27) | (1ul<<31);							// Enable terminal count interrupt
	ll[i].CTRL	= (	
					((size%2048)<<0) |	// Transfer size 
					(0<<12) |			// Source burst size 1
					(0<<15) |			// Dest burst size 1
					(0<<18) |			// 8bit source width
					(0<<21) | 			// 8bit dest width UART
					(1<<24) |			// Use AHB Master 1 for source transfer
					(1<<25) |			// Use AHB Master 1 for dest transfer
					(1<<26) |			// Enable source increment
					(0<<27) |			// Disable dest increment
					(1<<28) |			// Privileged mode 
					(1<<29) |			// Bufferable 
					(1UL<<31) 			// Terminal count interrupt enable.
					);


    LPC_GPDMA->INTTCCLEAR = 0x01;		// Clear status of channel 0
    LPC_GPDMA->INTERRCLR = 0x01;		// Clear errors on channel 0
	LPC_GPDMA->C0LLI = ll[0].LLI;		
    LPC_GPDMA->C0SRCADDR = ll[0].SRC; 		
    LPC_GPDMA->C0DESTADDR = ll[0].DST; 	
    LPC_GPDMA->C0CONTROL = ll[0].CTRL; 	
  	LPC_GPDMA->C0CONFIG = 	(1<<0)| 	// Channel enable
							(0<<0)|		// Source ignored with M2P
							(1<<6)|		// Destination = UART0 transmit
							(1<<11)|	// Memory to peripheral (DMA control) 
							(1<<14)| 	// Interrupt error mask
							(1<<15);  	// Terminal count interrupt mask.
}

/******************************************************************************
**                            End Of File
******************************************************************************/
