/*****************************************************************************
 * $Id: mc_comm.c 6577 2011-06-06 12:00:00Z nxp16893 $
 *
 * Project: MC_lib
 *
 * Description: Motor Control communication libraries
 *
 * Copyright(C) 2011, NXP Semiconductor
 * All rights reserved.
 *
 *****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#include "mc_inc.h"
#include "mc_comms.h"
#include "mc_lib.h"
#include "type.h"

uint32_t comms_time=0;			// Timer for connection timeout
uint32_t comms_timeout=0;		// Max timer for connection timeout

MC_COMMS_Type* p_comms;			// Local copy of pointer to global MC data structure


volatile uint8_t RxHead=0;
volatile uint8_t RxTail=0;

/*****************************************************************************
** Function name:		MC_COMM_Init
**
** Description:			Initialize MC_COMM parameters to default 
**
** Parameters:			None
** Returned value:		None 
*****************************************************************************/
void vMC_COMMS_Init(MC_COMMS_Type *comms, MC_DATA_Type *mc, uint32_t timeout)
{
	p_comms = comms;					// Save local copy of pointer to comms structure

	p_comms->tx_buf_full = FALSE;
	p_comms->tx_complete = TRUE;
	p_comms->trig.size=1;				// Number of trigger bytes to compare with
	p_comms->trig.edge=TRIGGER_NO_EDGE;	// Edge type of trigger
	p_comms->trig.address=0;				// Address in FOC struct of value to be triggered
	p_comms->trig.start=FALSE;			// Flag if signal condition prior to trigger transsision is met 
	p_comms->trig.done=FALSE;			// Flag if signal condition after trigger transsision is met 

	comms_timeout = timeout; 

	p_comms->sync.syncVAL[0] = 0xEF;
	p_comms->sync.syncVAL[1] = 0xF1;
	p_comms->sync.syncVAL[2] = 0x13;
	p_comms->sync.syncVAL[3] = 0x11;
	p_comms->sync.deviceID[0] = 0;
	p_comms->sync.deviceID[1] = 0;
	p_comms->sync.mc_lib_version = (MCLIB_V_MAJOR<<8) | MCLIB_V_MINOR;

	p_comms->ctrl.type = vMC_LIB_GetMCType();

#if defined MCLIB_TYPE_BLDC
	p_comms->sync.data_size = 	sizeof(CTRL_Type) + _
								sizeof(CALIB_Type) + _
								sizeof(BLDC_Type);
#elif defined MCLIB_TYPE_STEPPER
	/* tbd */
#elif defined MCLIB_TYPE_SVPWM
	/* tbd */
#elif defined MCLIB_TYPE_FOC_SYNC
	p_comms->sync.data_size =  	sizeof(CTRL_Type) + sizeof(CALIB_Type) + sizeof(FOC_Type);
#elif defined MCLIB_TYPE_FOC_ASYNC
	/* tbd */
#endif
				
	vMC_COMMS_ConnReset();
}	

void vMC_COMMS_SetTxBuffPointer()
{
	// TODO	
}

void vMC_COMMS_TimerFeed(uint32_t dt)
{
	if(comms_time)comms_time-=dt;
}


uint8_t u8MC_COMMS_IsConnected(void)
{
	
	return (comms_time>0)?1:0;
}

void vMC_COMMS_ConnReset(void)
{
	p_comms->sync.scope_size=0; 
	p_comms->sync.sync_size = sizeof(p_comms->sync);
	p_comms->sync.total_size =  p_comms->sync.sync_size + 
								p_comms->sync.data_size + 
 								p_comms->sync.scope_size + 
								2;
	p_comms->scope_list_cnt = 0; 
}

void vMC_COMMS_CreateTxBuff(MC_DATA_Type *mc)
{
	/* Copy ctrl struct, calib struct and data struct to the data buffer */
	memcpy(	(uint8_t *)&p_comms->ctrl, 
			(uint8_t *)&mc->ctrl, 
			p_comms->sync.data_size); 	

	p_comms->scope_list_index = 0; 	// Restart filling buffer. 
	p_comms->tx_buf_full=FALSE;		// Start rebuilding buffer
	p_comms->trig.start=FALSE;
	p_comms->trig.done=FALSE;

	/* End sync bytes */
	p_comms->scope[p_comms->sync.scope_size + 0]=0xDF;
	p_comms->scope[p_comms->sync.scope_size + 1]=0xFD;
}

void vMC_COMMS_ProcessChar(uint8_t ch)
{
	static uint8_t rxSeq=0;   			// Indicates whether first start char was received
	static uint8_t len=0;
	static uint8_t Data_Index=0;

	switch(rxSeq)
	{
		case(0):
			if(ch==RX_STARTCHAR)rxSeq++;	// Start character received
			break;
		case(1):
			if(ch<CMD_ALIVE_FEED)			// Invalid command? CMD_ALIVE_FEED = first command in enum
			{												   	
				rxSeq=0;					// Back to search
				break;
			}
			p_comms->rx_packets[RxHead].cmd = ch;		// Save command
			rxSeq++;
			break;
		case(2):
			len=ch;							// Length of remaining frame
			rxSeq++;						
			break;
		case(3):
			p_comms->rx_packets[RxHead].data[Data_Index++] = ch;	
			if(Data_Index==len)				// All data bytes in frame received + cmd
			{
				Data_Index=0;				// Reset Data Index
				rxSeq=0; 					// Wait for next package
				vMC_COMMS_SetNextPacketIndex();	
			}
			break;
		default: 
			rxSeq=0;  					
			break;
	}			
}

uint8_t u8MC_COMMS_GetRxPacketCnt(void)
{
	if(RxHead == RxTail)
	{
		return 0;
	}
	else if(RxHead > RxTail)
	{
		return RxHead - RxTail;
	}
	else
	{
		return RxHead + RX_MAX_PACKETS - RxTail;
	}
}

// Set next Queue index; used for enqueue
void vMC_COMMS_SetNextPacketIndex(void)
{
	RxHead++;			
	if(RxHead==RX_MAX_PACKETS)RxHead=0;
}

// Get next Queue index; used for dequeue
uint8_t u8MC_COMMS_GetNextPacketIndex(void)
{
	if(RxTail == RX_MAX_PACKETS-1)
	{
		RxTail = 0;
		return RX_MAX_PACKETS-1;		
	}
	else
	{
		RxTail++;
		return RxTail-1;	
	}
}

/*****************************************************************************
** Function name:		Process_Rx_Packet
**
** Description:			Processes received UART data
**
** Parameters:			None
** Returned value:		None
**
*****************************************************************************/
void vMC_COMMS_ProcessPacket(uint8_t index, MC_DATA_Type *mc)
{
	uint16_t u16FieldOffset;
	uint16_t u16Value;
	uint8_t u8BitNumber;
	uint8_t u8BitValue;
	uint8_t *p_u8;
	int16_t i16Tmp;
	RX_PACKET *packet;
	
	packet = &p_comms->rx_packets[index];

	switch(packet->cmd)
	{
 		case(CMD_ALIVE_FEED):			// Connection alive feed 
			comms_time = comms_timeout;
			break;
		case(CMD_CTRLEN):					// Enabled 
			/* Callback to Enable */
			if(p_comms->callbacks.Enable != NULL) 
			{
			  (*p_comms->callbacks.Enable)(packet->data[0]);
    		}
			break;
 		case(CMD_SPDCEN):				// SpeedControlEnabled 
		 	/* Callback to SpeedEnable */
			if(p_comms->callbacks.SpeedEnable != NULL) 
			{
			  (*p_comms->callbacks.SpeedEnable)(packet->data[0]);
    		}
			break;			
 		case(CMD_SETTXLIST):			// Set txlist items
			memcpy((uint8_t *)&p_comms->sync.scope_size, (uint8_t *)&packet->data[0], 2); 
			p_comms->sync.total_size =  p_comms->sync.sync_size + 
										p_comms->sync.data_size + 
 										p_comms->sync.scope_size + 
										2;
			p_comms->scope_list_cnt = packet->data[2];
			vMC_COMMS_CreateTxBuff(mc);	// Reset TxList
			// [0] = CMD
			// [1] = buffsize lo
			// [2] = buffsize hi
			// [3] = scope_list_cnt
			// [4..4+scope_list_cnt] = txlist
			
			// scope_list_cnt = number of scope items (16b addresses)
			// copy scope_list_cnt x 2 bytes of addresses
			memcpy((uint8_t *)&p_comms->scope_list, (uint8_t *)&packet->data[3], p_comms->scope_list_cnt*2); 
			break;
		case(CMD_SETTRIGGER):			// Set trigger 
			// RxFrame[0] 		= COMMAND
			// RxFrame[1] 		= trigger size
			// RxFrame[2] 		= trigger edge 
			// RxFrame[3]	    = trigger address in Comms struct 
			// RxFrame[4..7]	= trigger value
			p_comms->trig.size=packet->data[0];
			p_comms->trig.edge=packet->data[1];
			/* Copy trigger field offset */
			memcpy((uint8_t *)&p_comms->trig.address, (uint8_t *)&packet->data[2], 2); 
			/* Copy 32bit trigger value */
			memcpy((uint8_t *)&p_comms->trig.value, (uint8_t *)&packet->data[4], 4); 
			break;
		case(CMD_SAMPLE_POINT):			// New value for adc sample point 		
			/* data[0] = Sample number*/
			/* data[1..2] = point in PWM period */
			u16Value = ((uint16_t)packet->data[2] << 8) | packet->data[1];
			 /* Callback to SamplePoint */
			if(p_comms->callbacks.SamplePoint != NULL) 
			{
			  (*p_comms->callbacks.SamplePoint)(packet->data[0], u16Value);
    		}
			break;
		case(CMD_QEI_INV): 		// New value for QEI direction
			mc->calib.QEIinvert = packet->data[0];
			vQEI_SetDirection(mc->calib.QEIinvert);
			break;
		case(CMD_QEI_AUTOCAL): 	// Command to execute the QEI calibration routine		
			vMC_LIB_Disable(mc);
			vCALIB_QEICalib((CALIB_Type *)&mc->calib);
			vMC_LIB_Disable(mc);
			break;
		case(CMD_WRITE_BIT): 	// Command to write a single bit
			u16FieldOffset = ((uint16_t)packet->data[1] << 8) | packet->data[0];
			u8BitNumber = packet->data[2];
			u8BitValue = packet->data[3];

		   	// Get the pointer to the data; start of motor control data which is
			// the first byte of the CTRL structure in MC_DATA_Type	+ the field offset
			p_u8=(uint8_t *)&mc->ctrl + u16FieldOffset;
			// Write the new bitvalue on the pointer address
			if(u8BitValue)
			{
				/* Set bit */
				*p_u8 |= (1<<u8BitNumber);  
			}
			else
			{
				/* Clear bit */
				*p_u8 &= ~(1<<u8BitNumber);  
			}
			u8BitValue = mc->ctrl.brake_thresh;
			break;
		case(CMD_WRITE_8): 		// Command to write 8 bits
			u16FieldOffset = ((uint16_t)packet->data[1] << 8) | packet->data[0];
			/* Get the pointer to the data; start of motor control data which is
			   the first byte of the CTRL structure in MC_DATA_Type	+ the field offset */
			p_u8=(uint8_t *)&mc->ctrl + u16FieldOffset;
			/* Write 1 byte to the pointer address */
			memcpy(p_u8, (uint8_t *)&packet->data[2], 1); 
			break;
		case(CMD_WRITE_16): 	// Command to write 16 bits
			u16FieldOffset = ((uint16_t)packet->data[1] << 8) | packet->data[0];
			/* Get the pointer to the data; start of motor control data which is
			   the first byte of the CTRL structure in MC_DATA_Type	+ the field offset */
			p_u8=(uint8_t *)&mc->ctrl + u16FieldOffset;
			/* Write 2 bytes to the pointer address */
			memcpy(p_u8, (uint8_t *)&packet->data[2], 2); 
			break;									
		case(CMD_WRITE_32): 	// Command to write 32 bits
			u16FieldOffset = ((uint16_t)packet->data[1] << 8) | packet->data[0];
			/* Get the pointer to the data; start of motor control data which is
			   the first byte of the CTRL structure in MC_DATA_Type	+ the field offset */
			p_u8=(uint8_t *)&mc->ctrl + u16FieldOffset;
			/* Write 4 bytes to the pointer address */
			memcpy(p_u8, (uint8_t *)&packet->data[2], 4); 
			break;
		case(CMD_PID_KI):		// Command to set a PID KI
			u16FieldOffset = ((uint16_t)packet->data[1] << 8) | packet->data[0];				
			memcpy((uint8_t *)&i16Tmp, (uint8_t *)&packet->data[2], 2); 
			
			/* Get the pointer to the PID structure; start of motor control data which is
			   the first byte of the CTRL structure in MC_DATA_Type	+ the field offset.
			   Ki is 3rd value in PID structure: subtract 2*2=4 bytes */
			p_u8=(uint8_t *)&mc->ctrl + u16FieldOffset - 4;
			/* Set PID Ki value */			
			f6_10MC_LIB_PIDSetKi(i16Tmp, (F6_10_PID_Type*) p_u8);
		default: 				// Unknown command
			break;
	}
}

/*****************************************************************************
** Function name:		vMC_COMMS_AddTxBuffItems
**
** Description:			Trigger and collect data for the scope function
**
** Parameters:			pointer to Motor control data structure 
** Returned value:		None
**
*****************************************************************************/
void vMC_COMMS_AddTxBuffItems(MC_DATA_Type *mc)
{
	uint8_t i, *p, *src;
	uint32_t trig_sample=0;

	// Get pointer to start of motor control data, 
	// the first byte of the CTRL structure in MC_DATA_Type
	src = (uint8_t *)&mc->ctrl;
	
	// No data to put in buffer because the scope list count = 0
	if(p_comms->scope_list_cnt==0){
		// Set the buffer status to full
		p_comms->tx_buf_full = TRUE;
		return; 
	}
	
	// Check if tx buffer is already filled
	if(p_comms->tx_buf_full)return;

	// Start adding items to tx buffer
	// Get trigger sample from FOC
	memcpy((uint8_t *)&trig_sample, src+p_comms->trig.address, p_comms->trig.size);
	
	// If no trigger is found yet, try to find a trigger
	if(!p_comms->trig.done)	
	{
		switch(p_comms->trig.edge)
		{
			case(TRIGGER_NO_EDGE):
				p_comms->trig.done=TRUE;	
				break;
			case(TRIGGER_RISING_EDGE):
				// Compare it with the trig.value
				if(trig_sample<p_comms->trig.value)p_comms->trig.start=TRUE;				
				if(p_comms->trig.start && trig_sample>=p_comms->trig.value)p_comms->trig.done=TRUE;				
				break;
			case(TRIGGER_FALLING_EDGE):
				// Compare it with the trig.value
				if(trig_sample>p_comms->trig.value)p_comms->trig.start=TRUE;	
				if(p_comms->trig.start && trig_sample<p_comms->trig.value)p_comms->trig.done=TRUE;
				break;
		}
	}
	// A trigger has occurred, fill the buffer for the scope 
	else	
	{
		// Go through the scope list from i = 0 to its size
		for(i=0;i<p_comms->scope_list_cnt;i++)
		{
			// Get the pointer to the data; src offset + txlist
			p=src+p_comms->scope_list[i];
			// Now copy it to the scope buffer
			p_comms->scope[p_comms->scope_list_index++]= *p;	
		}

		// If the buffer index > maximum scope size 		
		if(p_comms->scope_list_index>=p_comms->sync.scope_size-1)
		{
			// Set the buffer status to full
			p_comms->tx_buf_full = TRUE;	
		}
	}
}
				 
void vMC_COMMS_Transmit(MC_DATA_Type *mc)
{
	// Check if a transmission has to be done, 
	// by checking if the previous transsmission is complete
	// and if there is an active connection
	if(p_comms->tx_complete && u8MC_COMMS_IsConnected())
	{
		p_comms->tx_complete = FALSE;
		// UART DMA transmission complete. Restart filling the buffer
		// Initialize tx buff: copy FOC params to data buff
		vMC_COMMS_CreateTxBuff(mc);			// Reset scope list and fill with data
		// Start UART DMA transmission directly. DMA on UART is slower then the FOC loop.
		///TODO: make this a more generic call, not specific for UART0
		vUART0_DMA_Send((uint32_t *)&p_comms->sync, p_comms->sync.total_size);
	}
}

void vMC_COMMS_SetTransmissionComplete(void)
{
	p_comms->tx_complete = TRUE;
}






/*****************************************************************************
**                            End Of File
******************************************************************************/
