/*****************************************************************************
 * $Id: mc_foc->c 6577 2011-06-06 12:00:00Z nxp16893 $
 *
 * Project: MC_lib
 *
 * Description:  Field Oriented Motor Control library
 *
 * Copyright(C) 2011, NXP Semiconductor
 * All rights reserved.
 *
 *****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#include "stdint.h" 
#include "app_setup.h"
#include "mc_foc.h"

MC_DATA_Type* pmc;			// Local copy of pointer to global MC data structure
FOC_Type* pfoc;				// Local copy of pointer to global FOC data structure

/*****************************************************************************
** Function name:		vMC_FOC_SetDefault
**
** Description:			Calculate and set default parameters for FOC
**
** Parameters:			pointer to parameter structure
** Returned value:		None 
*****************************************************************************/
void vMC_FOC_SetDefault(FOC_STORAGE_Type *stored)
{
	stored->F.full = F16_16_CONST(1 - PHASE_RESISTANCE * PWM_LOOPTIME / PHASE_INDUCTANCE);
	stored->G.full = F16_16_CONST(PWM_LOOPTIME / PHASE_INDUCTANCE);
	stored->K_smc.full = F16_16_CONST(K_SMC);
	stored->smc_max_err.full = F16_16_CONST(SMC_MAX_ERR);
	stored->Cflt_omega.full = F16_16_CONST(OMEGA0 / PWM_SPEEDRATE / 10); 		
}

void vMC_FOC_ImportStorage(FOC_STORAGE_Type *stored, FOC_Type *foc)
{
	/* Init FOC variables from saved parameters */
	vMC_LIB_PID_F6_10_Copy((F6_10_PID_Type *)&stored->pidQ, (F6_10_PID_Type *)&foc->pidQ);
	vMC_LIB_PID_F6_10_Copy((F6_10_PID_Type *)&stored->pidD, (F6_10_PID_Type *)&foc->pidD);
	
	foc->F.full = stored->F.full;
	foc->G.full = stored->G.full;
	foc->K_smc.full = stored->K_smc.full;
	foc->smc_max_err.full = stored->smc_max_err.full;
	foc->Cflt_omega.full = stored->Cflt_omega.full; 
}

/*****************************************************************************
** Function name:		FOC_Init
**
** Description:			Initialize FOC parameters to default 
**
** Parameters:			None
** Returned value:		None 
*****************************************************************************/
void vMC_FOC_Init(MC_DATA_Type *mc)
{
	pmc = mc;								// Save pointer to global MC data
	pfoc = &mc->var.foc;					// Save pointer to global FOC data 
	
	/* Initialize peripherals */	
	if(pmc->calib.QEIinvert)vQEI_SetDirection(1);

	pmc->ctrl.sensor_mode = MODE_QEI;
	pmc->ctrl.enabled = FALSE;
	pmc->ctrl.speedEna = FALSE; 

	/* Initialize common speed PID values */
	pmc->ctrl.pidSpeed.out_max.full = INT16_MAX;
	pmc->ctrl.pidSpeed.out_min.full = 0;
	vMC_LIB_PID_Reset(&pmc->ctrl.pidSpeed);
	
	/* Initialize FOC PID values */
	pfoc->pidQ.out_max.full = INT16_MAX;
	pfoc->pidQ.out_min.full = INT16_MIN;
	vMC_LIB_PID_Reset(&pfoc->pidQ);

	pfoc->pidD.out_max.full = INT16_MAX;
	pfoc->pidD.out_min.full = INT16_MIN;
	vMC_LIB_PID_Reset(&pfoc->pidD);
 	
	/* Initialize SMC */
	pfoc->F16_16_Valpha.full = F16_16_CONST(0.0);
	pfoc->bemf_alpha.full = F16_16_CONST(0.0);
	pfoc->bemf_alpha_flt.full = F16_16_CONST(0.0);
	pfoc->z_alpha.full = F16_16_CONST(0.0);
	pfoc->Ialpha_est.full = F16_16_CONST(0.0);
	pfoc->F16_16_Vbeta.full = F16_16_CONST(0.0);
	pfoc->bemf_beta.full = F16_16_CONST(0.0);
	pfoc->bemf_beta_flt.full = F16_16_CONST(0.0);
	pfoc->z_beta.full = F16_16_CONST(0.0);
	pfoc->Ibeta_est.full = F16_16_CONST(0.0);
	pfoc->F16_16_Ialpha.full = F16_16_CONST(0.0);
	pfoc->Ialpha_err.full = F16_16_CONST(0.0);
	pfoc->F16_16_Ibeta.full = F16_16_CONST(0.0);
	pfoc->Ibeta_err.full = F16_16_CONST(0.0);
	pfoc->theta.full = F16_16_CONST(0.0);
	pfoc->omega_est.full = F16_16_CONST(0.0);
}


/*****************************************************************************
** Function name:		vMC_FOC_Loop
**
** Description:			FOC loop containig the FOC algorithm
**
** Parameters:			None
** Returned value:		None 
*****************************************************************************/
int32_t i32tmp;
int32_t *ptmp;
void vMC_FOC_Loop(void)
{
	uint8_t i;
	F6_10 sinAlpha, cosAlpha;	// Sine and cosine component of alpha
	F6_10 Ibus;					// Bus current 
	F6_10 tmp;					// Temp variable
	F22_10 Vlim_square;			// Vector limit ^2
	int16_t oldQsp;				// Backup Q setpoint for current limit

	pmc->ctrl.busy = TRUE;
	
	/* Get phase current */ 	
	pfoc->ADCphase[0] = u16ADC_ReadDR(ADC_PHA_PORT, ADC_PHA_CH);
	pfoc->ADCphase[1] = u16ADC_ReadDR(ADC_PHB_PORT, ADC_PHB_CH);
	pfoc->ADCphase[2] = u16ADC_ReadDR(ADC_PHC_PORT, ADC_PHC_CH);

	for(i=0;i<3;i++)
	{
		pfoc->I[i].full = f6_10MC_LIB_ScaledADC(pfoc->ADCphase[i],
												 pmc->calib.IphADCoffset[i],
												 pmc->calib.IphADCgain[i]);
	}
	
	/* Get bus current */
	pfoc->ADCIbus = u16ADC_ReadDR(ADC_IBUS_PORT, ADC_IBUS_CH);
	Ibus.full = f6_10MC_LIB_ScaledADC(pfoc->ADCIbus, pmc->calib.IbusADCoffset, pmc->calib.IbusADCgain);

	/* Filter bus current */
	tmp.full = F6_10_CONST(C_FLT_IBUS);
	vMC_LIB_F6_10Filter(Ibus, &pfoc->Ibus, tmp);
	
	/* Clarke transformation */
	vMC_LIB_F6_10Clarke(pfoc->I[0], pfoc->I[1], &pfoc->Ialpha, &pfoc->Ibeta);

	/* Get QEI position	*/
	pfoc->QEIpos = u32QEI_Position();
	pfoc->alphaSens.full = f6_10MC_LIB_QEIToRad(&pmc->calib, pfoc->QEIpos);
	
#ifdef MCLIB_OPT_FOCSENSORLESS
	/* Get setimated position from virtual motor model */
	pfoc->alphaEst.full = f6_10MC_EstimateAngle();
#endif	
		
	/* Speed control */
	switch(pmc->ctrl.sensor_mode)
	{
		case(MODE_QEI_RAMP):
			/* tbd */
			break;		
#ifdef MCLIB_OPT_FOCSENSORLESS
		case(MODE_SL_RAMP):
			/* tbd */
			break;		
		case(MODE_SL):
			alphaUsed.full = pfoc->alphaEst.full;
			/* tbd */
			break;
#endif 
		case(MODE_QEI):

			if(u8QEI_IsVelocityUpdated())
			{	
				/* Get RPM from QEI */
				pmc->ctrl.RPM = i32QEI_RPM();
				/* Calculate rad/sec from RPM for comparison with sensorless algorithm */
				pfoc->omega.full = i32RPM_TO_F22_10RADS(pmc->ctrl.RPM);
				/* Check if speed PID is enabled */
				if(pmc->ctrl.speedEna)
				{  
					/* Execute speed PID */
					pfoc->pidQ.sp.full = i16MC_LIB_PID(i32RPM_TO_F6_10PID(pmc->ctrl.RPM), &pmc->ctrl.pidSpeed);	
				}
			}
			break;
	}

	/* Sine and cosine components from alpha */
	sinAlpha.full = F6_10_sin(pfoc->alphaSens.full);		
	cosAlpha.full = F6_10_cos(pfoc->alphaSens.full);		

	/* Park transformation */
	vMC_LIB_F6_10Park(pfoc->Ialpha, 
				 	  pfoc->Ibeta, 
				 	  &pfoc->Id, 
				 	  &pfoc->Iq, 
				 	  sinAlpha, 
				 	  cosAlpha);

	/* Current limit Q setpoint */ 
	oldQsp=pfoc->pidQ.sp.full;
	vf6_10MC_LIB_CurrentLimitQsp(&pfoc->pidQ.sp, &pfoc->Vq, &pfoc->Vd, &pfoc->Ibus);
	 	
	/* Q & D PI loops */
	pfoc->Vq.full = f6_10MC_LIB_PID(pfoc->Iq, &pfoc->pidQ);
	pfoc->Vd.full = f6_10MC_LIB_PID(pfoc->Id, &pfoc->pidD);	

	/* Restore original Q setpoint before current limit */ 
	pfoc->pidQ.sp.full=oldQsp;

	// The Vq/Vd vector can be maximum one. 
	// Limit Vq, Vd is not limited
	// SQRT(Vd^2 + VqMax^2) < 31.999
	// VqMax = SQRT(31.999^2 - Vd^2)
#ifdef MCLIB_OPT_SINGLESHUNT
	// Single shunt
 	// The pwm vector is limited to be able to measure current through the shunt resistor.
	Vlim_square.full = F22_10_CONST((32*PWM_MAX)*(32*PWM_MAX));
#else
	// Multi shunt
 	// When Valpha and VBeta are calculated, a slight rounding error can occur, 
	// cousing the total vector of Valpha and VBeta to be slightly larger then 1.00
	// If we limit the vector at this point to 1020.0 in stead of 1023.9375 = (32.999^2) the rounding .
	// errors can never occur.
	Vlim_square.full = F22_10_CONST(1020.0); 
#endif 

	/* Limit current by limiting Q vector */
	vf6_10MC_LIB_LimitVq(Vlim_square, &pfoc->Vq, &pfoc->Vd, &pfoc->Ibus);

	//pfoc->Vq.full = F6_10_CONST(3.0); 
	//pfoc->Vd.full = F6_10_CONST(0.0); 

	/* Inverse park : translate static Vq and Vd to dynamic Valpha and Vbeta */
	vMC_LIB_F6_10InversePark(pfoc->Vd, pfoc->Vq, &pfoc->Valpha, &pfoc->Vbeta, sinAlpha, cosAlpha);
	
	/* Inverse Clarke */
	vMC_LIB_F6_10InverseClarke(pfoc->Valpha, pfoc->Vbeta, &pfoc->Vr1, &pfoc->Vr2, &pfoc->Vr3);
		
	/* Scale from 32 to 1; divide by 32 (>>5) */
	pfoc->Vr1.full >>=5; 
	pfoc->Vr2.full >>=5; 
	pfoc->Vr3.full >>=5; 

	/* Calculate pwm with SVPWM algorithm */
	vMC_LIB_F6_10SVPWM(pfoc->Vr1, pfoc->Vr2, pfoc->Vr3, &pfoc->pwm, &pmc->ctrl.sector);
	
	if(pmc->ctrl.enabled)vMC_LIB_SetPWM_F6_10(&pfoc->pwm);
	vMC_COMMS_AddTxBuffItems(pmc);		
	vMC_COMMS_Transmit(pmc);
	
	pmc->ctrl.busy = FALSE;
	return;
}

void f6_10MC_Commutate(int16_t duty, int16_t alpha)
{
	F6_10 sinAlpha, cosAlpha;	// Sine and cosine component of alpha
	F6_10 Vq, Vd, Valpha, Vbeta, Vr1, Vr2, Vr3;
	F6_10_PWM_Type pwm;
	uint8_t sector;

	/* Sine and cosine components from alpha */
	sinAlpha.full = F6_10_sin(alpha);		
	cosAlpha.full = F6_10_cos(alpha);		

	/* Set vectors */
	Vq.full = duty;		
	Vd.full = F6_10_CONST(0.0); 

	/* Inverse park : translate static Vq and Vd to dynamic Valpha and Vbeta */
	vMC_LIB_F6_10InversePark(Vd, Vq, &Valpha, &Vbeta, sinAlpha, cosAlpha);
	
	/* Inverse Clarke */
	vMC_LIB_F6_10InverseClarke(Valpha, Vbeta, &Vr1, &Vr2, &Vr3);
		
	/* Scale from 32 to 1; divide by 32 (>>5) */
	Vr1.full >>=5; 
	Vr2.full >>=5; 
	Vr3.full >>=5; 

	/* Calculate pwm with SVPWM algorithm */
	vMC_LIB_F6_10SVPWM(Vr1, Vr2, Vr3, &pwm, &sector);
	
	/* Set PWM output */
	vMC_LIB_SetPWM_F6_10(&pwm);
}

/*****************************************************************************
** Function name:		f6_10MC_EstimateAngle
**
** Description:			Virtual motor model estimating stator angle
**
** Parameters:			None
** Returned value:		estimated angle 
*****************************************************************************/
int16_t f6_10MC_EstimateAngle(void)
{
	/* tbd */
	return 0;
}

/*****************************************************************************
**                            End Of File
******************************************************************************/
